<?php

namespace Zen\Paymentpage\Block;

use Magento\Checkout\Helper\Cart;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\App\ResponseFactory;
use Magento\Framework\Registry;
use Magento\Framework\UrlInterface;
use Magento\Framework\View\Element\Template;
use Magento\Framework\View\Element\Template\Context;
use Magento\Store\Api\Data\StoreInterface;
use Zen\Payment\CartData;
use Zen\Payment\Paywall;
use Zen\Payment\Util;
use Zen\Paymentpage\Model\Order;

/**
 * Class Checkout
 *
 * @package Zen\Paymentpage\Block
 */
class Checkout extends Template
{

	/**
	 * @var \Magento\Framework\UrlInterface
	 */
	public $urlBuilder;

	/**
	 * @var \Magento\Framework\App\Config\ScopeConfigInterface
	 */
	protected $scopeConfig;

	/**
	 * @var Registry
	 */
	private $registry;

	/**
	 * @var Order
	 */
	private $order;

	/**
	 * @var StoreInterface
	 */
	private $store;

	/**
	 * @var Cart
	 */
	private $cartHelper;

	/**
	 * @var ResponseFactory
	 */
	private $responseFactory;

	/**
	 * Checkout constructor.
	 *
	 * @param Context              $context
	 * @param Order                $order
	 * @param Registry             $registry
	 * @param StoreInterface       $store
	 * @param UrlInterface         $urlBuilder
	 * @param ScopeConfigInterface $scopeConfig
	 * @param Cart                 $cartHelper
	 * @param ResponseFactory      $responseFactory
	 *
	 * @internal param OrderFactory $orderFactory
	 */
	public function __construct(
		Context $context,
		Order $order,
		Registry $registry,
		StoreInterface $store,
		UrlInterface $urlBuilder,
		ScopeConfigInterface $scopeConfig,
		Cart $cartHelper,
		ResponseFactory $responseFactory

	) {
		parent::__construct($context);
		$this->registry = $registry;
		$this->order = $order;
		$this->store = $store;
		$this->urlBuilder = $urlBuilder;
		$this->scopeConfig = $scopeConfig;
		$this->cartHelper = $cartHelper;
		$this->responseFactory = $responseFactory;

		if((isset($_POST['orderId']) && $_POST['orderId']) && ($this->getOrderState($this->getOrderIdFromPost()) === \Magento\Sales\Model\Order::STATE_PENDING_PAYMENT)) {

			$this->redirectToHomePage();
			die();
		}
	}

	/**
	 * @param string $orderId
	 *
	 * @return mixed
	 */
	public function getOrderState($orderId = '')
	{
		return $this->getOrderObject($orderId)->getState();
	}

	/**
	 * Render form with order information prepared for Payment Page
	 *
	 * @param string $id
	 *
	 * @return bool|\Zen\Paymentpage\Model\Sales\Order
	 */
	public function getOrderObject($id = '')
	{
		if(empty($id)) {
			$id = $this->getOrderId();
		}

		return $this->order->loadOrderById($id);
	}

	/**
	 * @return integer
	 */
	public function getOrderId()
	{
		return $this->registry->registry('orderId');
	}

	/**
	 * @return string
	 */
	public function getOrderIdFromPost()
	{
		$id = '';
		if(isset($_POST['orderId'])) {
			$id = $_POST['orderId'];
		}

		return $id;
	}

	/**
	 * That function redirect to homepage
	 *
	 * @return mixed
	 */
	public function redirectToHomePage()
	{
		return $this->responseFactory->create()->setRedirect($this->urlBuilder->getUrl())->sendResponse(); //Redirect to cms page
	}

	/**
	 * @param string $orderId
	 *
	 * @return string
	 */
	public function renderForm($orderId = '')
	{

		if(empty($orderId)) {
			$orderId = $this->getOrderId();
		}

		$order = $this->getOrderObject($orderId);

		if(!$order) {
			$this->redirectToHomePage();
			die();
		}

		$order->setState(\Magento\Sales\Model\Order::STATE_PENDING_PAYMENT);
		$order->save();

		$methodInstance = $order->getPayment()->getMethodInstance();

		$paywall = new Paywall(
			$this->getEnvironment(),
			$this->scopeConfig->getValue('payment/zen_paymentpage/paywall_secret'),
			$this->scopeConfig->getValue('payment/zen_paymentpage/terminal_id')
		);

		$orderTotal = Util::convertAmountToFractional($order->getGrandTotal());

		$orderData = Paywall::prepareData(
			$orderTotal,
			$order->getOrderCurrency()->getCode(),
			$order->getIncrementId(),
			$order->getBillingAddress()->getFirstname(),
			$order->getBillingAddress()->getLastname(),
			$order->getCustomerEmail(),
			$this->urlBuilder->getUrl('checkout/onepage/success/'),
			$this->urlBuilder->getUrl('checkout/onepage/failure/'),
			$this->urlBuilder->getUrl('checkout/onepage/failure/'),
			$this->urlBuilder->getUrl('zen_paymentpage/payment/notification'),
			$this->getCart($order, $orderTotal)
		);

		return $paywall->buildOrderForm($orderData, __('Continue'), strtolower(substr($methodInstance->getLocale(), 0, 2)));
	}

	/**
	 * @return string
	 */
	public function getEnvironment()
	{

		return Util::ENVIRONMENT_PRODUCTION;
	}

	/**
	 * @param Order $order
	 * @param int   $orderTotal
	 *
	 * @return array
	 */
	private function getCart($order, $orderTotal)
	{

		$cartData = new CartData();

		$cartData->setAmount($orderTotal);

		foreach($order->getAllItems() as $item) {

			$itemCost = Util::convertAmountToFractional($item->getPriceInclTax());
			$quantity = (int) $item->getQtyOrdered();

			$cartData->addItem(
				$item->getName(),
				$itemCost,
				$quantity,
				$itemCost * $quantity
			);
		}

		$shippingDescription = $order->getShippingDescription();

		if(!$order->getIsVirtual() && $shippingDescription){
			$shippingCost = Util::convertAmountToFractional($order->getShippingInclTax());

			$cartData->addItem(
				$shippingDescription,
				$shippingCost,
				1,
				$shippingCost
			);
		}

		$discount = Util::convertAmountToFractional($order->getBaseDiscountAmount());

		if($discount !== 0) {
			$cartData->addItem(
				__('Discount'),
				$discount,
				1,
				$discount
			);
		}

		return $cartData->getCartData();
	}
}
